//-----------------------------------------------------------------------------
//     The confidential and proprietary information contained in this file may
//     only be used by a person authorised under and to the extent permitted
//     by a subsisting licensing agreement from ARM Limited or its affiliates.
//
//            (C) COPYRIGHT 2015-2020 ARM Limited or its affiliates.
//                ALL RIGHTS RESERVED
//
//     This entire notice must be reproduced on all copies of this file
//     and copies of this file may only be made by a person if such person is
//     permitted to do so under the terms of a subsisting license agreement
//     from ARM Limited or its affiliates.
//
//     Release Information : ARES-MP051-r4p1-00rel0
//
//-----------------------------------------------------------------------------
// SystemVerilog (IEEE Std 1800-2012)
//-----------------------------------------------------------------------------






`include "ares_header.sv"


module ares_vx_crypt
(


  input wire            clk,
  input wire            reset,
  

  input wire            ival_v1_q,
  input wire            sha1c_v1_q,      
  input wire            sha1p_v1_q,      
  input wire            sha1m_v1_q,      
  input wire            sha1cpm_v1_q,    
  input wire            sha256h_v1_q,    
  input wire            sha256h2_v1_q,   
  input wire            sha256hh2_v1_q,  
  
  input wire            aesd_v1_q,       
  input wire            aese_v1_q,       
  input wire            aesd_or_e_v1_q,  
  input wire            aesmc_v1_q,      
  input wire            aesimc_v1_q,     
  input wire            aesdimc_v1_q,    
  input wire            aesemc_v1_q,     
  input wire            pmull_v1_q,      
  input wire            sha1h_v1_q,      
  input wire            sha1su0_v1_q,    
  input wire            sha1su1_v1_q,    
  input wire            sha256su0_v1_q,  
  input wire            sha256su1_v1_q,  

  input wire            sha256su1_dup_x_v1_q,  
  input wire            sha256su1_dup_y_v1_q,  
  input wire            sha256su1_dup_z_v1_q,  

  input wire [127:0]    opa,             
  input wire [127:0]    opb,             
  input wire [127:0]    opc,             

  output wire [127:0]   cryptout_v2,     
  output wire [127:0]   cryptout_v4,     
  output wire           crypt_active
);







  wire [15:0]                            aes_shf_v1;
  wire [127:0]                           aesd_out;
  wire [15:0]                            aesd_shf_v1;
  wire [127:0]                           aesd_v1;
  reg                                    aesd_v2_q;
  reg                                    aesdimc_h_v2_q;
  reg                                    aesdimc_l_v2_q;
  wire [127:0]                           aesdimc_out;
  reg                                    aesdimc_v2_q;
  wire [127:0]                           aese_out;
  wire [15:0]                            aese_shf_v1;
  wire [127:0]                           aese_v1;
  reg                                    aese_v2_q;
  wire [127:0]                           aesed_lut_in_v1;
  wire [127:0]                           aesed_lut_out_v1;
  reg                                    aesemc_h_v2_q;
  reg                                    aesemc_l_v2_q;
  wire [127:0]                           aesemc_out;
  reg                                    aesemc_v2_q;
  reg                                    aesimc_h_v2_q;
  reg                                    aesimc_l_v2_q;
  wire [127:0]                           aesimc_out;
  reg                                    aesimc_v2_q;
  reg                                    aesmc_h_v2_q;
  reg                                    aesmc_l_v2_q;
  wire [127:0]                           aesmc_out;
  reg                                    aesmc_v2_q;
  wire                                   block_opa_passthrough;
  wire                                   ival_en;
  wire                                   ival_v2_4latency;
  reg                                    ival_v2_q;
  reg                                    ival_v3_q;
  reg                                    ival_v4_q;
  wire [127:0]                           newa_v1;
  wire [127:0]                           newa_v2;
  wire [127:0]                           newa_v3;
  wire [127:0]                           newb_v1;
  wire [127:0]                           newb_v2;
  wire [127:0]                           newb_v3;
  wire [127:0]                           newx_v4;
  wire [127:0]                           newy_v4;
  wire [127:0]                           opa_pmull_v1;
  reg  [127:0]                           opa_v2_q;
  reg  [127:0]                           opa_v3_q;
  reg  [127:0]                           opa_v4_q;
  reg  [127:0]                           opb_v2_q;
  reg  [127:0]                           opb_v3_q;
  reg  [127:0]                           opb_v4_q;
  reg  [95:0]                            opc_v2_q;
  reg  [63:0]                            opc_v3_q;
  reg  [31:0]                            opc_v4_q;
  reg                                    pmull_h_v2_q;
  reg                                    pmull_l_v2_q;
  wire [127:0]                           pmull_out_v1;
  reg                                    pmull_v2_q;
  wire [127:0]                           qx_v1;
  reg                                    sel_aesd_h_v2_q;
  reg                                    sel_aesd_l_v2_q;
  wire                                   sel_aesd_v1;
  reg                                    sel_aese_h_v2_q;
  reg                                    sel_aese_l_v2_q;
  wire                                   sel_aese_v1;
  wire [127:0]                           sha1_xin_v1;
  wire [31:0]                            sha1_yin_v1;
  wire [31:0]                            sha1_zin_v1;
  reg                                    sha1c_v2_q;
  reg                                    sha1c_v3_q;
  reg                                    sha1c_v4_q;
  reg                                    sha1cpm_h_v4_q;
  reg                                    sha1cpm_l_v4_q;
  reg                                    sha1cpm_v2_q;
  reg                                    sha1cpm_v3_q;
  reg                                    sha1cpm_v4_q;
  wire [127:0]                           sha1cpm_x_v1;
  wire [127:0]                           sha1cpm_x_v2;
  wire [127:0]                           sha1cpm_x_v3;
  wire [127:0]                           sha1cpm_x_v4;
  wire [127:0]                           sha1cpm_y_v1;
  wire [127:0]                           sha1cpm_y_v2;
  wire [127:0]                           sha1cpm_y_v3;
  wire [31:0]                            sha1cpm_y_v4;
  wire [31:0]                            sha1h_in_v1;
  wire [31:0]                            sha1h_out_v1;
  reg                                    sha1m_v2_q;
  reg                                    sha1m_v3_q;
  reg                                    sha1m_v4_q;
  reg                                    sha1p_v2_q;
  reg                                    sha1p_v3_q;
  reg                                    sha1p_v4_q;
  reg                                    sha1su0_h_v2_q;
  reg                                    sha1su0_l_v2_q;
  wire [127:0]                           sha1su0_out_v1;
  reg                                    sha1su0_v2_q;
  wire [127:0]                           sha1su1_out_v1;
  wire [127:0]                           sha1su1_qdin_bar_v1;
  wire [127:0]                           sha1su1_qdin_v1;
  wire [127:0]                           sha1su1_qnin_bar_v1;
  wire [127:0]                           sha1su1_qnin_v1;
  wire [127:0]                           sha256_xin_v1;
  wire [127:0]                           sha256_yin_v1;
  wire [31:0]                            sha256_zin_v1;
  reg                                    sha256h2_h_v4_q;
  reg                                    sha256h2_l_v4_q;
  reg                                    sha256h2_v2_q;
  reg                                    sha256h2_v3_q;
  reg                                    sha256h_h_v4_q;
  reg                                    sha256h_l_v4_q;
  reg                                    sha256h_v2_q;
  reg                                    sha256h_v3_q;
  wire [127:0]                           sha256h_x_v1;
  wire [127:0]                           sha256h_x_v2;
  wire [127:0]                           sha256h_x_v3;
  wire [127:0]                           sha256h_x_v4;
  wire [127:0]                           sha256h_y_v1;
  wire [127:0]                           sha256h_y_v2;
  wire [127:0]                           sha256h_y_v3;
  wire [127:0]                           sha256h_y_v4;
  wire                                   sha256hh2_v2;
  wire [127:0]                           sha256su0_out_v1;
  wire [127:0]                           sha256su0_qdin_v1;
  wire [127:0]                           sha256su0_qnin_v1;
  reg                                    sha256su1_dup_x_v2_q;
  reg                                    sha256su1_dup_y_v2_q;
  reg                                    sha256su1_dup_z_v2_q;
  reg                                    sha256su1_h_v2_q;
  reg                                    sha256su1_l_v2_q;
  reg                                    sha256su1_v2_q;
  wire [63:0]                            sha256su1_x_v1;
  wire [63:0]                            sha256su1_x_v2;
  reg                                    sha_inst_h_v2_q;
  reg                                    sha_inst_l_v2_q;
  wire                                   sha_inst_v1;
  reg                                    sha_inst_v2_q;
  wire                                   short_pipe_out_v3_en;
  wire [31:0]                            sigma0_v3;
  wire [31:0]                            sigma0_v4;
  wire [31:0]                            sigma1_v3;
  wire [31:0]                            tchoose_v3;
  wire [31:0]                            tmajority_v3;
  wire [31:0]                            tmajority_v4;
  wire [127:0]                           xORy_v4;
  wire [32:0]                            x_fa2_c_v4;
  wire [31:0]                            x_fa2_s_v4;
  wire [127:0]                           x_v1;
  wire [127:0]                           x_v2;
  wire [127:0]                           x_v3;
  wire [127:0]                           x_v4;
  wire                                   xprime_carry;
  wire [127:96]                          xprime_v4;
  wire [32:0]                            xy_fa0_c_v3;
  wire [31:0]                            xy_fa0_s_v3;
  wire [32:0]                            xy_fa1_c_v3;
  wire [31:0]                            xy_fa1_c_v4;
  wire [31:0]                            xy_fa1_s_v3;
  wire [31:0]                            xy_fa1_s_v4;
  wire [32:0]                            y_fa2_c_v4;
  wire [31:0]                            y_fa2_s_v4;
  wire [32:0]                            y_fa3_c_v4;
  wire [31:0]                            y_fa3_s_v4;
  wire [127:0]                           y_v1;
  wire [127:0]                           y_v2;
  wire [127:0]                           y_v3;
  wire [127:0]                           y_v4;
  wire                                   yprime_carry;
  wire [127:96]                          yprime_v4;
  wire [127:0]                           z_v1;
  wire [95:0]                            z_v2;
  wire [63:0]                            z_v3;
  wire [31:0]                            z_v4;

  

assign ival_en = ival_v1_q | ival_v2_q | ival_v3_q | ival_v4_q;

  always_ff @(posedge clk or posedge reset)
  begin: u_ival_v2_q
    if (reset == 1'b1)
      ival_v2_q <= `ARES_DFF_DELAY {1{1'b0}};
`ifdef ARES_XPROP_FLOP
    else if (reset == 1'b0 && ival_en == 1'b1)
      ival_v2_q <= `ARES_DFF_DELAY ival_v1_q;
    else if (reset == 1'b0 && ival_en == 1'b0)
    begin
    end
    else
      ival_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
`else
    else if (ival_en == 1'b1)
      ival_v2_q <= `ARES_DFF_DELAY ival_v1_q;
`endif
  end


  always_ff @(posedge clk or posedge reset)
  begin: u_ival_v3_q
    if (reset == 1'b1)
      ival_v3_q <= `ARES_DFF_DELAY {1{1'b0}};
`ifdef ARES_XPROP_FLOP
    else if (reset == 1'b0 && ival_en == 1'b1)
      ival_v3_q <= `ARES_DFF_DELAY ival_v2_4latency;
    else if (reset == 1'b0 && ival_en == 1'b0)
    begin
    end
    else
      ival_v3_q <= `ARES_DFF_DELAY {1{1'bx}};
`else
    else if (ival_en == 1'b1)
      ival_v3_q <= `ARES_DFF_DELAY ival_v2_4latency;
`endif
  end


  always_ff @(posedge clk or posedge reset)
  begin: u_ival_v4_q
    if (reset == 1'b1)
      ival_v4_q <= `ARES_DFF_DELAY {1{1'b0}};
`ifdef ARES_XPROP_FLOP
    else if (reset == 1'b0 && ival_en == 1'b1)
      ival_v4_q <= `ARES_DFF_DELAY ival_v3_q;
    else if (reset == 1'b0 && ival_en == 1'b0)
    begin
    end
    else
      ival_v4_q <= `ARES_DFF_DELAY {1{1'bx}};
`else
    else if (ival_en == 1'b1)
      ival_v4_q <= `ARES_DFF_DELAY ival_v3_q;
`endif
  end


assign short_pipe_out_v3_en = aesd_v2_q   | aese_v2_q     | aesmc_v2_q  | aesimc_v2_q
                            | aesdimc_v2_q| aesemc_v2_q   | pmull_v2_q  | sha_inst_v2_q
                            | sha1su0_v2_q| sha256su1_v2_q;

assign ival_v2_4latency = ~short_pipe_out_v3_en & ival_v2_q;

assign sel_aesd_v1 = aesd_v1_q & ~aesdimc_v1_q;
assign sel_aese_v1 = aese_v1_q & ~aesemc_v1_q;


  always_ff @(posedge clk)
  begin: u_sha1c_v2_q_grp
    if (ival_v1_q == 1'b1) begin
      sha1c_v2_q <= `ARES_DFF_DELAY sha1c_v1_q;
      sha1p_v2_q <= `ARES_DFF_DELAY sha1p_v1_q;
      sha1m_v2_q <= `ARES_DFF_DELAY sha1m_v1_q;
      sha256h_v2_q <= `ARES_DFF_DELAY sha256h_v1_q;
      sha256h2_v2_q <= `ARES_DFF_DELAY sha256h2_v1_q;
      sha1cpm_v2_q <= `ARES_DFF_DELAY sha1cpm_v1_q;
      aesd_v2_q <= `ARES_DFF_DELAY aesd_v1_q;
      aese_v2_q <= `ARES_DFF_DELAY aese_v1_q;
      aesmc_v2_q <= `ARES_DFF_DELAY aesmc_v1_q;
      aesmc_h_v2_q <= `ARES_DFF_DELAY aesmc_v1_q;
      aesmc_l_v2_q <= `ARES_DFF_DELAY aesmc_v1_q;
      aesimc_v2_q <= `ARES_DFF_DELAY aesimc_v1_q;
      aesimc_h_v2_q <= `ARES_DFF_DELAY aesimc_v1_q;
      aesimc_l_v2_q <= `ARES_DFF_DELAY aesimc_v1_q;
      aesdimc_v2_q <= `ARES_DFF_DELAY aesdimc_v1_q;
      aesdimc_h_v2_q <= `ARES_DFF_DELAY aesdimc_v1_q;
      aesdimc_l_v2_q <= `ARES_DFF_DELAY aesdimc_v1_q;
      aesemc_v2_q <= `ARES_DFF_DELAY aesemc_v1_q;
      aesemc_h_v2_q <= `ARES_DFF_DELAY aesemc_v1_q;
      aesemc_l_v2_q <= `ARES_DFF_DELAY aesemc_v1_q;
      pmull_v2_q <= `ARES_DFF_DELAY pmull_v1_q;
      pmull_h_v2_q <= `ARES_DFF_DELAY pmull_v1_q;
      pmull_l_v2_q <= `ARES_DFF_DELAY pmull_v1_q;
      sha_inst_v2_q <= `ARES_DFF_DELAY sha_inst_v1;
      sha_inst_h_v2_q <= `ARES_DFF_DELAY sha_inst_v1;
      sha_inst_l_v2_q <= `ARES_DFF_DELAY sha_inst_v1;
      sha1su0_v2_q <= `ARES_DFF_DELAY sha1su0_v1_q;
      sha1su0_h_v2_q <= `ARES_DFF_DELAY sha1su0_v1_q;
      sha1su0_l_v2_q <= `ARES_DFF_DELAY sha1su0_v1_q;
      sha256su1_v2_q <= `ARES_DFF_DELAY sha256su1_v1_q;
      sha256su1_h_v2_q <= `ARES_DFF_DELAY sha256su1_v1_q;
      sha256su1_l_v2_q <= `ARES_DFF_DELAY sha256su1_v1_q;
      sel_aesd_h_v2_q <= `ARES_DFF_DELAY sel_aesd_v1;
      sel_aesd_l_v2_q <= `ARES_DFF_DELAY sel_aesd_v1;
      sel_aese_h_v2_q <= `ARES_DFF_DELAY sel_aese_v1;
      sel_aese_l_v2_q <= `ARES_DFF_DELAY sel_aese_v1;
      sha256su1_dup_x_v2_q <= `ARES_DFF_DELAY sha256su1_v1_q;
      sha256su1_dup_y_v2_q <= `ARES_DFF_DELAY sha256su1_v1_q;
      sha256su1_dup_z_v2_q <= `ARES_DFF_DELAY sha256su1_v1_q;
    end
`ifdef ARES_XPROP_FLOP
    else if (ival_v1_q == 1'b0)
    begin
    end
    else begin
      sha1c_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha1p_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha1m_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha256h_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha256h2_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha1cpm_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      aesd_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      aese_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      aesmc_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      aesmc_h_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      aesmc_l_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      aesimc_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      aesimc_h_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      aesimc_l_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      aesdimc_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      aesdimc_h_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      aesdimc_l_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      aesemc_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      aesemc_h_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      aesemc_l_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      pmull_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      pmull_h_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      pmull_l_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha_inst_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha_inst_h_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha_inst_l_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha1su0_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha1su0_h_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha1su0_l_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha256su1_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha256su1_h_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha256su1_l_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sel_aesd_h_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sel_aesd_l_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sel_aese_h_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sel_aese_l_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha256su1_dup_x_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha256su1_dup_y_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha256su1_dup_z_v2_q <= `ARES_DFF_DELAY {1{1'bx}};
    end
`endif
  end



  always_ff @(posedge clk)
  begin: u_sha1c_v3_q_grp
    if (ival_v2_4latency == 1'b1) begin
      sha1c_v3_q <= `ARES_DFF_DELAY sha1c_v2_q;
      sha1p_v3_q <= `ARES_DFF_DELAY sha1p_v2_q;
      sha1m_v3_q <= `ARES_DFF_DELAY sha1m_v2_q;
      sha1cpm_v3_q <= `ARES_DFF_DELAY sha1cpm_v2_q;
      sha256h_v3_q <= `ARES_DFF_DELAY sha256h_v2_q;
      sha256h2_v3_q <= `ARES_DFF_DELAY sha256h2_v2_q;
    end
`ifdef ARES_XPROP_FLOP
    else if (ival_v2_4latency == 1'b0)
    begin
    end
    else begin
      sha1c_v3_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha1p_v3_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha1m_v3_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha1cpm_v3_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha256h_v3_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha256h2_v3_q <= `ARES_DFF_DELAY {1{1'bx}};
    end
`endif
  end



  always_ff @(posedge clk)
  begin: u_sha1c_v4_q_grp
    if (ival_v3_q == 1'b1) begin
      sha1c_v4_q <= `ARES_DFF_DELAY sha1c_v3_q;
      sha1p_v4_q <= `ARES_DFF_DELAY sha1p_v3_q;
      sha1m_v4_q <= `ARES_DFF_DELAY sha1m_v3_q;
      sha1cpm_v4_q <= `ARES_DFF_DELAY sha1cpm_v3_q;
      sha1cpm_h_v4_q <= `ARES_DFF_DELAY sha1cpm_v3_q;
      sha1cpm_l_v4_q <= `ARES_DFF_DELAY sha1cpm_v3_q;
      sha256h_h_v4_q <= `ARES_DFF_DELAY sha256h_v3_q;
      sha256h_l_v4_q <= `ARES_DFF_DELAY sha256h_v3_q;
      sha256h2_h_v4_q <= `ARES_DFF_DELAY sha256h2_v3_q;
      sha256h2_l_v4_q <= `ARES_DFF_DELAY sha256h2_v3_q;
    end
`ifdef ARES_XPROP_FLOP
    else if (ival_v3_q == 1'b0)
    begin
    end
    else begin
      sha1c_v4_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha1p_v4_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha1m_v4_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha1cpm_v4_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha1cpm_h_v4_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha1cpm_l_v4_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha256h_h_v4_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha256h_l_v4_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha256h2_h_v4_q <= `ARES_DFF_DELAY {1{1'bx}};
      sha256h2_l_v4_q <= `ARES_DFF_DELAY {1{1'bx}};
    end
`endif
  end




ares_vx_sha256su1 u_sha256su1_v1(
  .sha256su1_x_op   (sha256su1_dup_x_v1_q),
  .sha256su1_y_op   (sha256su1_dup_y_v1_q),
  .sha256su1_z_op   (sha256su1_dup_z_v1_q),

  .x                (opc[63:0]),               
  .y                (opa[95:32]),              
  .z                (opb[127:64]),             
  .newx             (sha256su1_x_v1[63:0])
);

assign qx_v1[127:0] = {128{aesd_or_e_v1_q}} & (opb[127:0] ^ opa[127:0]);
  
ares_vx_aese1 u_aese1(
        .q              (qx_v1[127:0]),
        .aese_out       (aese_v1[127:0]),
        .aese_shf       (aese_shf_v1[15:0])
);

ares_vx_aesd1 u_aesd1(
        .q              (qx_v1[127:0]),
        .aesd_out       (aesd_v1[127:0]),
        .aesd_shf       (aesd_shf_v1[15:0])
);

  ares_nand_gate #(.WIDTH(32)) u_sha1h_in_v1_31_0 (.vec_o(sha1h_in_v1[31:0]), .cmn_i(sha1h_v1_q), .vec_i(opa[31:0]));

ares_vx_sha1h u_sha1h(
        .qn             (~sha1h_in_v1[31:0]),
        .d              (sha1h_out_v1[31:0])
);


  ares_nand_gate #(.WIDTH(128)) u_sha1su1_qdin_bar_v1_127_0 (.vec_o(sha1su1_qdin_bar_v1[127:0]), .cmn_i(sha1su1_v1_q), .vec_i(opb[127:0]));
  ares_nand_gate #(.WIDTH(128)) u_sha1su1_qnin_bar_v1_127_0 (.vec_o(sha1su1_qnin_bar_v1[127:0]), .cmn_i(sha1su1_v1_q), .vec_i(opa[127:0]));

assign sha1su1_qdin_v1[127:0] = ~sha1su1_qdin_bar_v1[127:0];
assign sha1su1_qnin_v1[127:0] = ~sha1su1_qnin_bar_v1[127:0];

ares_vx_sha1su1 u_sha1su1(
        .qd             (sha1su1_qdin_v1[127: 0]),
        .qn             (sha1su1_qnin_v1[127:32]),
        .d              (sha1su1_out_v1[127: 0])
);

assign sha256su0_qdin_v1[127:0] = {128{sha256su0_v1_q}} & opb[127:0];
assign sha256su0_qnin_v1[127:0] = {128{sha256su0_v1_q}} & opa[127:0];

ares_vx_sha256su0 u_sha256su0(
        .qd             (sha256su0_qdin_v1[127:0]),
        .qn             (sha256su0_qnin_v1[ 31:0]),
        .d              (sha256su0_out_v1[127:0])
);

assign sha_inst_v1 = sha1h_v1_q | sha1su1_v1_q | sha256su0_v1_q;

assign aes_shf_v1[15:0] = {16{aese_v1_q}} & aese_shf_v1[15:0] |
                          {16{aesd_v1_q}} & aesd_shf_v1[15:0];

assign sha1su0_out_v1 [127:0] = opc[127:0] ^ {opa[63:0], opc[127:64]} ^ opb[127:0];

assign x_v1[127:0] = opc[127:0]; 
assign y_v1[127:0] = opa[127:0];
assign z_v1[127:0] = opb[127:0];

  ares_nand_gate #(.WIDTH(128)) u_sha1_xin_v1_127_0 (.vec_o(sha1_xin_v1[127:0]), .cmn_i(sha1cpm_v1_q), .vec_i(x_v1[127:0]));
  ares_nand_gate #(.WIDTH(32)) u_sha1_yin_v1_31_0 (.vec_o(sha1_yin_v1[31:0]), .cmn_i(sha1cpm_v1_q), .vec_i(y_v1[31:0]));
  ares_nand_gate #(.WIDTH(32)) u_sha1_zin_v1_31_0 (.vec_o(sha1_zin_v1[31:0]), .cmn_i(sha1cpm_v1_q), .vec_i(z_v1[31:0]));

ares_vx_sha1cpm u_sha1cpm_v1(
  .choose           (sha1c_v1_q),
  .parity           (sha1p_v1_q),
  .majority         (sha1m_v1_q),
  .cpm              (sha1cpm_v1_q),
  .x                (~sha1_xin_v1[127:0]),
  .y                (~sha1_yin_v1[31:0]),
  .z                (~sha1_zin_v1[31:0]),
  .newx             (sha1cpm_x_v1[127:0]),
  .newy             (sha1cpm_y_v1[31:0])
);
assign sha1cpm_y_v1[127:32] = {96{sha1cpm_v1_q}} & y_v1[127:32]; 

  ares_nand_gate #(.WIDTH(128)) u_sha256_xin_v1_127_0 (.vec_o(sha256_xin_v1[127:0]), .cmn_i(sha256hh2_v1_q), .vec_i(x_v1[127:0]));
  ares_nand_gate #(.WIDTH(128)) u_sha256_yin_v1_127_0 (.vec_o(sha256_yin_v1[127:0]), .cmn_i(sha256hh2_v1_q), .vec_i(y_v1[127:0]));
  ares_nand_gate #(.WIDTH(32)) u_sha256_zin_v1_31_0 (.vec_o(sha256_zin_v1[31:0]), .cmn_i(sha256hh2_v1_q), .vec_i(z_v1[31:0]));

ares_vx_sha256h32 u_sha256h32_v1(
  .x                (~sha256_xin_v1[127:0]),
  .y                (~sha256_yin_v1[127:0]),
  .z                (~sha256_zin_v1[31:0]),
  .newx             (sha256h_x_v1[127:0]),
  .newy             (sha256h_y_v1[127:0])
);

assign block_opa_passthrough = sha1su0_v1_q | sha256su1_v1_q | sha256hh2_v1_q | sha1cpm_v1_q | aesd_or_e_v1_q | sha_inst_v1 | pmull_v1_q;

assign aesed_lut_in_v1[127:0] =  ({128{aese_v1_q}} & aese_v1[127:0]) | ({128{aesd_v1_q}} & aesd_v1[127:0]);
ares_vx_aesed2_lut u_aesed2_lut_v1(
  .lut_in           (aesed_lut_in_v1[127:0]),
  .lut_out          (aesed_lut_out_v1[127:0])
);

assign newb_v1[127:0] = ({128{sha1cpm_v1_q  }}  & sha1cpm_x_v1[127:0])                
                      | ({128{sha256hh2_v1_q}}  & sha256h_x_v1[127:0])                
                      | ({128{sha256su1_v1_q}}  & {opc[127:64], sha256su1_x_v1[63:0]})
                      | ({128{aesd_or_e_v1_q}}  & {{112{1'b0}},aes_shf_v1[15:0]})     
                      | ({128{  sha1su0_v1_q}}  & {sha1su0_out_v1 [127:0]});          

assign newa_v1[127:0] = ({128{sha1cpm_v1_q  }}  & sha1cpm_y_v1[127:0])                
                      | ({128{sha256hh2_v1_q}}  & sha256h_y_v1[127:0])                
                      | ({128{sha256su1_v1_q}}  & {opb[31:0],opa[127:32]})            
                      | ({128{sha1h_v1_q}}      & {{96{1'b0}}, sha1h_out_v1[31:0]})   
                      | ({128{sha1su1_v1_q}}    & sha1su1_out_v1[127:0])              
                      | ({128{sha256su0_v1_q}}  & sha256su0_out_v1[127:0])            
                      | ({128{aesd_or_e_v1_q}}  & aesed_lut_out_v1[127:0])            
                      | ({128{pmull_v1_q}}      & pmull_out_v1[127:0])                
                      | ({128{~(block_opa_passthrough)}} & opa[127:0]);




  always_ff @(posedge clk)
  begin: u_opb_v2_q_127_0_grp
    if (ival_v1_q == 1'b1) begin
      opb_v2_q[127:0] <= `ARES_DFF_DELAY newb_v1[127:0];
      opa_v2_q[127:0] <= `ARES_DFF_DELAY newa_v1[127:0];
      opc_v2_q[95:0] <= `ARES_DFF_DELAY opb[127:32];
    end
`ifdef ARES_XPROP_FLOP
    else if (ival_v1_q == 1'b0)
    begin
    end
    else begin
      opb_v2_q[127:0] <= `ARES_DFF_DELAY {128{1'bx}};
      opa_v2_q[127:0] <= `ARES_DFF_DELAY {128{1'bx}};
      opc_v2_q[95:0] <= `ARES_DFF_DELAY {96{1'bx}};
    end
`endif
  end



ares_vx_sha256su1 u_sha256su1_v2(
  .sha256su1_x_op   (sha256su1_dup_x_v2_q),
  .sha256su1_y_op   (sha256su1_dup_y_v2_q),
  .sha256su1_z_op   (sha256su1_dup_z_v2_q),

  .x                (x_v2[127:64]),            
  .y                (y_v2[127:64]),            
  .z                (x_v2[63:0]),              
  .newx             (sha256su1_x_v2[63:0])
);

ares_vx_aesed2 u_aesed2(
        .clk           (clk),
        .ival_v1_q     (ival_v1_q),
        .aes_din_v1    (aesed_lut_out_v1[127:0]),
        .aes_shf_v1    (aes_shf_v1[15:0]),
        .aesd_out      (aesd_out[127:0]),
        .aese_out      (aese_out[127:0]),
        .aesemc_out    (aesemc_out[127:0]),
        .aesdimc_out   (aesdimc_out[127:0])
);

ares_vx_aesmc u_aesmc(
        .d_in          (opa_v2_q[127:0]),
        .mc            (aesmc_out[127:0])
);

ares_vx_aesimc u_aesimc(
        .d_in          (opa_v2_q[127:0]),
        .imc           (aesimc_out[127:0])
);


assign opa_pmull_v1[127:0] = {opb[63:0], opa[63:0]};

ares_vx_pmull u_pmull_v1(
        .a_in          (opa_pmull_v1[63:0]),
        .b_in          (opa_pmull_v1[127:64]),
        .p_out         (pmull_out_v1[127:0])
);

assign x_v2[127:0] = opb_v2_q[127:0];
assign y_v2[127:0] = opa_v2_q[127:0];
assign z_v2[95:0]  = opc_v2_q[95:0];
ares_vx_sha1cpm u_sha1cpm_v2(
  .choose           (sha1c_v2_q),
  .parity           (sha1p_v2_q),
  .majority         (sha1m_v2_q),
  .cpm              (sha1cpm_v2_q),
  .x                (x_v2[127:0]),
  .y                (y_v2[31:0]),
  .z                (z_v2[31:0]),
  .newx             (sha1cpm_x_v2[127:0]),
  .newy             (sha1cpm_y_v2[31:0])
);
assign sha1cpm_y_v2[127:32] = y_v2[127:32]; 
ares_vx_sha256h32 u_sha256h32_v2(
  .x                (x_v2[127:0]),
  .y                (y_v2[127:0]),
  .z                (z_v2[31:0]),
  .newx             (sha256h_x_v2[127:0]),
  .newy             (sha256h_y_v2[127:0])
);

assign sha256hh2_v2 = sha256h_v2_q | sha256h2_v2_q;
assign newb_v2[127:0] = ({128{sha1cpm_v2_q  }} & sha1cpm_x_v2[127:0])
                      | ({128{sha256hh2_v2  }} & sha256h_x_v2[127:0]);

assign newa_v2[127:0] = ({128{sha1cpm_v2_q  }} & sha1cpm_y_v2[127:0])
                      | ({128{sha256hh2_v2  }} & sha256h_y_v2[127:0]);

assign cryptout_v2[127:64] =  ({64{sha256su1_h_v2_q}}       & sha256su1_x_v2[63:0])
                            | ({64{sel_aesd_h_v2_q}}        & aesd_out[127:64])
                            | ({64{sel_aese_h_v2_q}}        & aese_out[127:64])
                            | ({64{aesmc_h_v2_q}}           & aesmc_out[127:64])
                            | ({64{aesemc_h_v2_q}}          & aesemc_out[127:64])
                            | ({64{aesimc_h_v2_q}}          & aesimc_out[127:64])
                            | ({64{aesdimc_h_v2_q}}         & aesdimc_out[127:64])
                            | ({64{sha_inst_h_v2_q}}        & opa_v2_q[127:64])
                            | ({64{pmull_h_v2_q}}           & opa_v2_q[127:64])
                            | ({64{sha1su0_h_v2_q}}         & opb_v2_q[127:64]);


assign cryptout_v2[63:0] =    ({64{sha256su1_l_v2_q}}       & opb_v2_q[63:0])
                            | ({64{sel_aesd_l_v2_q}}        & aesd_out[63:0])
                            | ({64{sel_aese_l_v2_q}}        & aese_out[63:0])
                            | ({64{aesmc_l_v2_q}}           & aesmc_out[63:0])
                            | ({64{aesemc_l_v2_q}}          & aesemc_out[63:0])
                            | ({64{aesimc_l_v2_q}}          & aesimc_out[63:0])
                            | ({64{aesdimc_l_v2_q}}         & aesdimc_out[63:0])
                            | ({64{sha_inst_l_v2_q}}        & opa_v2_q[63:0])
                            | ({64{pmull_l_v2_q}}           & opa_v2_q[63:0])
                            | ({64{sha1su0_l_v2_q}}         & opb_v2_q[63:0]);




  always_ff @(posedge clk)
  begin: u_opb_v3_q_127_0_grp
    if (ival_v2_4latency == 1'b1) begin
      opb_v3_q[127:0] <= `ARES_DFF_DELAY newb_v2[127:0];
      opa_v3_q[127:0] <= `ARES_DFF_DELAY newa_v2[127:0];
      opc_v3_q[63:0] <= `ARES_DFF_DELAY opc_v2_q[95:32];
    end
`ifdef ARES_XPROP_FLOP
    else if (ival_v2_4latency == 1'b0)
    begin
    end
    else begin
      opb_v3_q[127:0] <= `ARES_DFF_DELAY {128{1'bx}};
      opa_v3_q[127:0] <= `ARES_DFF_DELAY {128{1'bx}};
      opc_v3_q[63:0] <= `ARES_DFF_DELAY {64{1'bx}};
    end
`endif
  end



assign x_v3[127:0] = opb_v3_q[127:0];
assign y_v3[127:0] = opa_v3_q[127:0];
assign z_v3[63:0] =  opc_v3_q[63:0];

ares_vx_sha1cpm u_sha1cpm_v3(
  .choose           (sha1c_v3_q),
  .parity           (sha1p_v3_q),
  .majority         (sha1m_v3_q),
  .cpm              (sha1cpm_v3_q),
  .x                (x_v3[127:0]),
  .y                (y_v3[31:0]),
  .z                (z_v3[31:0]),
  .newx             (sha1cpm_x_v3[127:0]),
  .newy             (sha1cpm_y_v3[31:0])
);
assign sha1cpm_y_v3[127:32] = y_v3[127:32]; 

ares_vx_sha256h32 u_sha256h32_v3(
  .x                (x_v3[127:0]),
  .y                (y_v3[127:0]),
  .z                (z_v3[31:0]),
  .newx             (sha256h_x_v3[127:0]),
  .newy             (sha256h_y_v3[127:0])
);


  assign tchoose_v3[31:0] = (sha256h_y_v3[31:0] & sha256h_y_v3[63:32]) | 
                            (~sha256h_y_v3[31:0] & sha256h_y_v3[95:64]);

  assign tmajority_v3[31:0] = (sha256h_x_v3[31:0] & sha256h_x_v3[63:32]) |
                              (sha256h_x_v3[31:0] & sha256h_x_v3[95:64]) | 
                              (sha256h_x_v3[63:32] & sha256h_x_v3[95:64]);

  assign sigma0_v3[31:0] = {sha256h_x_v3[1:0],  sha256h_x_v3[31:2]}    
                         ^ {sha256h_x_v3[12:0], sha256h_x_v3[31:13]}   
                         ^ {sha256h_x_v3[21:0], sha256h_x_v3[31:22]};  

  assign sigma1_v3[31:0] = {sha256h_y_v3[5:0],  sha256h_y_v3[31:6]}    
                         ^ {sha256h_y_v3[10:0], sha256h_y_v3[31:11]}   
                         ^ {sha256h_y_v3[24:0], sha256h_y_v3[31:25]};  



  assign xy_fa0_s_v3[31:0] = sha256h_y_v3[127:96] ^ z_v3[63:32] ^ tchoose_v3[31:0];

  assign xy_fa0_c_v3[32:0] = {sha256h_y_v3[127:96] & z_v3[63:32] | tchoose_v3[31:0] & 
                             (sha256h_y_v3[127:96] | z_v3[63:32]), 1'b0};



  assign xy_fa1_s_v3[31:0] = xy_fa0_s_v3[31:0] ^ xy_fa0_c_v3[31:0] ^ sigma1_v3[31:0];

  assign xy_fa1_c_v3[32:0] = {xy_fa0_s_v3[31:0] & xy_fa0_c_v3[31:0] | sigma1_v3[31:0] & 
                             (xy_fa0_s_v3[31:0] | xy_fa0_c_v3[31:0]), 1'b0};


assign newb_v3[127:0] = ({128{sha1cpm_v3_q}} & sha1cpm_x_v3[127:0])
                      | ({128{sha256h_v3_q  }} & sha256h_x_v3[127:0]) 
                      | ({128{sha256h2_v3_q }} & {sha256h_x_v3[127:96], sha256h_y_v3[95:0]} );
 

assign newa_v3[127:0] = ({128{sha1cpm_v3_q}} & sha1cpm_y_v3[127:0])
                      | ({128{sha256h_v3_q}} & {sigma0_v3[31:0],tmajority_v3[31:0],
                                                xy_fa1_s_v3[31:0],xy_fa1_c_v3[31:0]})

                      | ({128{sha256h2_v3_q}} & {sigma0_v3[31:0],tmajority_v3[31:0],
                                                 xy_fa1_s_v3[31:0],xy_fa1_c_v3[31:0]});



  always_ff @(posedge clk)
  begin: u_opb_v4_q_127_0_grp
    if (ival_v3_q == 1'b1) begin
      opb_v4_q[127:0] <= `ARES_DFF_DELAY newb_v3[127:0];
      opa_v4_q[127:0] <= `ARES_DFF_DELAY newa_v3[127:0];
      opc_v4_q[31:0] <= `ARES_DFF_DELAY opc_v3_q[63:32];
    end
`ifdef ARES_XPROP_FLOP
    else if (ival_v3_q == 1'b0)
    begin
    end
    else begin
      opb_v4_q[127:0] <= `ARES_DFF_DELAY {128{1'bx}};
      opa_v4_q[127:0] <= `ARES_DFF_DELAY {128{1'bx}};
      opc_v4_q[31:0] <= `ARES_DFF_DELAY {32{1'bx}};
    end
`endif
  end



assign x_v4[127:0] = opb_v4_q[127:0];
assign y_v4[127:0] = opa_v4_q[127:0];
assign z_v4[31:0] =  opc_v4_q[31:0];

ares_vx_sha1cpm u_sha1cpm_v4(
  .choose           (sha1c_v4_q),
  .parity           (sha1p_v4_q),
  .majority         (sha1m_v4_q),
  .cpm              (sha1cpm_v4_q),
  .x                (x_v4[127:0]),
  .y                (y_v4[31:0]),
  .z                (z_v4[31:0]),
  .newx             (sha1cpm_x_v4[127:0]),
  .newy             (sha1cpm_y_v4[31:0])
);


  assign xORy_v4[127:0] =  x_v4[127:0];

  assign sigma0_v4[31:0]    = y_v4[127:96];
  assign tmajority_v4[31:0] = y_v4[95:64];
  assign xy_fa1_s_v4[31:0]  = y_v4[63:32];
  assign xy_fa1_c_v4[31:0]  = y_v4[31:0];


  assign x_fa2_s_v4[31:0] = xy_fa1_s_v4[31:0] ^ xy_fa1_c_v4[31:0] ^ xORy_v4[127:96];

  assign x_fa2_c_v4[32:0] = {xy_fa1_s_v4[31:0] & xy_fa1_c_v4[31:0] |
                             xORy_v4[127:96] & (xy_fa1_s_v4[31:0] | 
                             xy_fa1_c_v4[31:0]), 1'b0};



  assign y_fa2_s_v4[31:0] = sigma0_v4[31:0] ^ tmajority_v4[31:0] ^ xy_fa1_c_v4[31:0];

  assign y_fa2_c_v4[32:0] = {sigma0_v4[31:0] & tmajority_v4[31:0] | 
                             xy_fa1_c_v4[31:0] & (sigma0_v4[31:0] | 
                             tmajority_v4[31:0]), 1'b0};



  assign y_fa3_s_v4[31:0] = y_fa2_s_v4[31:0] ^ y_fa2_c_v4[31:0] ^ xy_fa1_s_v4[31:0];

  assign y_fa3_c_v4[32:0] = {y_fa2_s_v4[31:0] & y_fa2_c_v4[31:0] | 
                             xy_fa1_s_v4[31:0] & (y_fa2_s_v4[31:0] | 
                             y_fa2_c_v4[31:0]), 1'b0};



  assign {xprime_carry, xprime_v4[127:96]} = x_fa2_s_v4[31:0] + x_fa2_c_v4[31:0] + {{31{1'b0}}, 1'b0};


  assign {yprime_carry, yprime_v4[127:96]} = y_fa3_s_v4[31:0] + y_fa3_c_v4[31:0] + {{31{1'b0}}, 1'b0};

  
  assign newx_v4[127:0] = {xORy_v4[95:0], yprime_v4[127:96]};
  assign newy_v4[127:0] = {xORy_v4[95:0], xprime_v4[127:96]};

  assign sha256h_x_v4[127:0] = newx_v4[127:0];
  assign sha256h_y_v4[127:0] = newy_v4[127:0];


assign cryptout_v4[63:0] = ({64{sha1cpm_l_v4_q}}   & sha1cpm_x_v4[63:0])
                         | ({64{sha256h_l_v4_q}}   & sha256h_x_v4[63:0])
                         | ({64{sha256h2_l_v4_q}}  & sha256h_y_v4[63:0]);

assign cryptout_v4[127:64] = ({64{sha1cpm_h_v4_q}}   & sha1cpm_x_v4[127:64])
                           | ({64{sha256h_h_v4_q}}   & sha256h_x_v4[127:64])
                           | ({64{sha256h2_h_v4_q}}  & sha256h_y_v4[127:64]);


assign crypt_active  = (ival_v1_q |
                        ival_v2_q |
                        ival_v3_q |
                        ival_v4_q
                        );

endmodule


`define ARES_UNDEFINE
`include "ares_header.sv"
`undef ARES_UNDEFINE
